#!/bin/sh
# backup-afs-drupal.sh v.1.2 by Sharon L. Krossa
# 
# Backs up Drupal directory and database 
# to DrupalBackups in top level of group or dept AFS account
#
# Written by Sharon L. Krossa, skrossa@sharonkrossa.com & skrossa@stanford.edu
# v. 1.2 (4 Dec 2010)
#        Fixed Drupal commands to be compatible with Drupal 3 (no spaces)
# v. 1.1 (26 Apr 2010)
#        Fixed so archiveNamePart1 also created when argument used for extraLabel
# v. 1.0 (11 Apr 2010)
# 
#
# Optional argument (for extra label for archive file names)
#
# Need to use from corn.stanford.edu in order to use Drush
#
# Must be run from within top level directory of drupal site
# e.g., /afs/ir/group/drupaltraining/cgi-bin/drupaltraining
#
# Not suitable for sites (including multisites) that use more than one database
# 
# TO DO: remove spaces from interactively entered extra label

########################################################################
# Error checking function                                              #
########################################################################
# The exit status is stored in the shell variable $?.
# So usage syntax is
# check_errs $? "${redf}Sorry, problem! Exiting.${reset}"
check_errs()
{
  # Function. Parameter 1 is the return code
  # Para. 2 is text to display on failure.
  if [ "${1}" -ne "0" ] ; then
    echo "ERROR # ${1} : ${2}"
    # as a bonus, make our script exit with the right error code.
    exit ${1}
  fi
}
########################################################################
# Command execution and feedback function                              #
########################################################################
# usage syntax is:
# do_and_feedback "drush -v dl drupal" "download Drupal"

do_and_feedback()
{
 curCommand=$1
 if [ "$2" ] ; then
   extraFeedback=$2
   echo "${greenf}Okay, ${extraFeedback} by doing ${cyanf}${curCommand}${reset}"
 else
   echo "${greenf}Okay, doing ${cyanf}${curCommand}${reset}"
 fi
 $curCommand
 check_errs $? "${redf}Sorry, problem with ${cyanf}${curCommand} ${redf}-- Exiting.${reset}"
}

########################################################################
# Set variables for easy color inclusion in echos                      #
########################################################################
# from Wicked Cool Shell Scripts written by Dave Taylor
# http://www.intuitive.com/wicked/showscript.cgi?011-colors.sh
# ANSI Color -- use these variables to easily have different color
#    and format output. Make sure to output the reset sequence after 
#    colors (f = foreground, b = background), and use the 'off'
#    feature for anything you turn on.

initializeANSI()
{
# apparently the funny character esc is set to works when run
  esc=""
  
  blackf="${esc}[30m";   redf="${esc}[31m";    greenf="${esc}[32m"
  yellowf="${esc}[33m"   bluef="${esc}[34m";   purplef="${esc}[35m"
  cyanf="${esc}[36m";    whitef="${esc}[37m"
  
  blackb="${esc}[40m";   redb="${esc}[41m";    greenb="${esc}[42m"
  yellowb="${esc}[43m"   blueb="${esc}[44m";   purpleb="${esc}[45m"
  cyanb="${esc}[46m";    whiteb="${esc}[47m"

  boldon="${esc}[1m";    boldoff="${esc}[22m"
  italicson="${esc}[3m"; italicsoff="${esc}[23m"
  ulon="${esc}[4m";      uloff="${esc}[24m"
  invon="${esc}[7m";     invoff="${esc}[27m"

  reset="${esc}[0m"
}

initializeANSI

########################################################################
# main script starts here                                              #
########################################################################

# Check that script being run from a main Drupal directory
# by checking for certain Drupal files

if [ ! -f "index.php" ] ; then
  echo "${redf}Sorry, ${mainPath} doesn't appear to be a main Drupal directory (no index.php file). Exiting.${reset}"
  exit
elif [ ! -f "install.php" ] ; then
  echo "${redf}Sorry, ${mainPath} doesn't appear to be a main Drupal directory (no install.php file). Exiting.${reset}"
  exit
elif [ ! -f "update.php" ] ; then
  echo "${redf}Sorry, ${mainPath} doesn't appear to be a main Drupal directory (no update.php file). Exiting.${reset}"
  exit
elif [ "grep --count --regexp='Drupal' index.php" '<' '1' ] ; then
  echo "${redf}Sorry, ${mainPath} doesn't appear to be a main Drupal directory ('Drupal' not in index.php). Exiting.${reset}"
  exit
elif [ "grep --count --regexp='Drupal' install.php" '<' '1' ] ; then
  echo "${redf}Sorry, ${mainPath} doesn't appear to be a main Drupal directory ('Drupal' not in install.php). Exiting.${reset}"
  exit
elif [ "grep --count --regexp='Drupal' update.php" '<' '1' ] ; then
  echo "${redf}Sorry, ${mainPath} doesn't appear to be a main Drupal directory ('Drupal' not in update.php). Exiting.${reset}"
  exit
else
  echo "${greenf}Confirmed: this is a main Drupal directory. Continuing.${reset}"
fi

# Parse path

mainPath=$PWD

origIFS=$IFS
IFS="/"
i=0
for pdir in $mainPath ; do
  i=`expr $i + 1`
  case "$i" in
    ( 1 ) pnothing=$pdir;;
    ( 2 ) afs=$pdir;;
    ( 3 ) ir=$pdir;;
    ( 4 ) deptOrGroup=$pdir;;
    ( 5 ) deptOrGroupDir=$pdir;;
    ( 6 ) cgiBin=$pdir;;
    ( 7 ) drupalDir=$pdir;;
    ( 8 ) endDir=$pdir;;
  esac
done
IFS=$origIFS

# Check whether directory is immediately below cgi-bin 
# (part 1, checking number of directories in path)

if [ $endDir ] ; then
  echo "${redf}Sorry, this script is only designed to work with Drupal directories right under the cgi-bin directory. Exiting.${reset}"
  exit
fi

# Check whether directory is in Stanford AFS space

stanfordPath="/${afs}/${ir}"
#echo "stanfordPath is ${stanfordPath}"

if [ "$stanfordPath" = "/afs/ir" ] ; then
  echo "${greenf}Confirmed: in Stanford AFS space. Continuing.${reset}"
elif [ "$stanfordPath" = "/afs/ir.stanford.edu" ] ; then
  echo "${greenf}Confirmed: in Stanford AFS space. Continuing.${reset}"
else
  echo "${redf}Sorry, this script is only designed to work in Stanford AFS space. Exiting.${reset}"
  exit
fi

# Check whether directory is in a group or dept account

if [ "$deptOrGroup" = "group" ] ; then
  echo "${greenf}Confirmed: Stanford AFS 'group' account. Continuing.${reset}"
elif [ "$deptOrGroup" = "dept" ] ; then
  echo "${greenf}Confirmed: Stanford AFS 'dept' account. Continuing.${reset}"
else
  echo "${redf}Sorry, this script is only designed to work in Stanford AFS 'dept' or 'group' accounts. Exiting.${reset}"
  exit 
fi

# Check whether directory is under cgi-bin directory (step 2)

if [ "$cgiBin" = "cgi-bin" ] ; then
  echo "${greenf}Confirmed: Drupal directory within cgi-bin directory. Continuing.${reset}"
else
  echo "${redf}Sorry, in Stanford AFS space, Drupal directories must be in the cgi-bin directory. Exiting.${reset}"
  exit
fi

# Establish variables for backup directory

backupDir="DrupalBackups"
backupDirPath="${stanfordPath}/${deptOrGroup}/${deptOrGroupDir}/${backupDir}"
# relBackupDir is the location of backupDir relative to mainPath
relBackupDirPath="../../DrupalBackups"

# Establish variable for date label

dateLabel=`date +%Y-%m-%d-%H%M%S`

# Get extra label, if any, from arguments, otherwise ask for
# (optional) extra label, and establish first part of file names

if [ "$#" -gt "0" ] ; then
  extraLabel=$1
  archiveNamePart1="${drupalDir}-${dateLabel}-${extraLabel}-drupal"
else
  echo "${boldon}${bluef}Enter extra label for .sql and directory archive file names (e.g., \"sitename\" or \"preupdate\") If none, leave blank:${reset}${boldoff}"
  read extraLabel
  if [ $extraLabel ] ; then
    echo "${greenf}Okay, extra label is \"${extraLabel}\"${reset}"
    archiveNamePart1="${drupalDir}-${dateLabel}-${extraLabel}-drupal"
  else
    extraLabel=""
    echo "${greenf}Okay, no extra label.${reset}"
    archiveNamePart1="${drupalDir}-${dateLabel}-drupal"
  fi
fi

# Establish full file names

filesArchiveName="${archiveNamePart1}-dir.tar.gz"
databaseArchiveName="${archiveNamePart1}-db.sql"

# Check for backup directory, and if not there, create

echo "Checking for ${backupDirPath} directory..."
if [ -d "$backupDirPath" ] ; then # do nothing, DrupalBackups dir exists
  echo "${greenf}It's there!${reset}"
else # make directory
  do_and_feedback "mkdir ${backupDirPath}" "make DrupalBackups directory"
fi

# Check whether files archive of same name already exists

echo "Checking if ${backupDirPath}/${filesArchiveName} already exists..."
if [ -f "${backupDirPath}/${filesArchiveName}" ] ; then
  echo "${redf}${backupDirPath}/${filesArchiveName} already exists! Exiting.${reset}"
  exit
else
  echo "${greenf}It doesn't!${reset}"
fi

# Backup Drupal directory
# Done from immediately above directory, so no extra directories in paths in archive

do_and_feedback "cd .." "change directory to just above ${drupalDir}"
do_and_feedback "tar -vczf ${backupDirPath}/${filesArchiveName} ${drupalDir}" "backup Drupal directory"
do_and_feedback "cd ${drupalDir}" "change directory back to ${drupalDir}"

# Check whether database archive of same name already exists

echo "Checking if  ${backupDirPath}/${databaseArchiveName} already exists..."
if [ -f "${backupDirPath}/${databaseArchiveName}" ] ; then
  echo "${redf}${backupDirPath}/${databaseArchiveName} already exists! Exiting.${reset}"
  exit
else
  echo "${greenf}It doesn't!${reset}"
fi

# Backup database (database dump)

do_and_feedback "drush -v sql-dump --result-file=${backupDirPath}/${databaseArchiveName}" "backup Drupal database"

# Note successful completion.

echo ""
echo "${boldon}${greenf}Backup of ${deptOrGroupDir} ${drupalDir} Drupal directory and database complete.${reset}${boldoff}"
echo ""